/*
 * Types/enumerations for [Atom] DBO investigation programs
 *
 *	Copyright (C) 2000, Rick Gorton
 *
 *	Written by Rick Gorton, July, 2000
 *	gorton@world.std.com
 *	All rights reserved
 *
 * NOTE:
 *	If you find this tool useful, the author is interested in knowing
 *	how you used it, and what kind of modifications you found useful.
 *
 */

// #define USE_INT_BLOCK_IDS 1

/*
 *	Controls for kinds of information to collect and report
 */
#define DUMP_PROC_INFO          0x1
#define DUMP_BB_INFO            0x2
#define DUMP_PATH_AS_BB_ID      0x4
#define DUMP_PATH_AS_BB_ADDR    0x8
#define DUMP_PATH_SOURCE_INFO   0x10
#define DUMP_EVERYTHING         DUMP_PROC_INFO | DUMP_BB_INFO | DUMP_PATH_AS_BB_ID |\
                                DUMP_PATH_AS_BB_ADDR | DUMP_PATH_SOURCE_INFO

#define DUMP_OUTPUT_MODE        (DUMP_BB_INFO | DUMP_PATH_AS_BB_ID)


#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

typedef unsigned char	U8;
typedef unsigned char	u8;
typedef u8	BOOLEAN;

typedef union {
		long	longstuff;
		double	doublestuff;
		struct {
			unsigned long	mantissa:52;
			unsigned long	exponent:11;
			unsigned long	sign:1;
			} rawbits;
		} IEEE_UNION_;

enum FloatPrecision_Enum {
	UNINITIALIZED_PREC_STATE,
	SINGLE_PREC_STATE,
	DOUBLE_PREC_STATE,
	ERROR_PREC_STATE
	};

#define	NUM_FP_REGS_	32
enum FloatPrecision_Enum	FP_RegState[NUM_FP_REGS_];

typedef struct {
	enum FloatPrecision_Enum	curr_state;
	enum Opcode			last_set_opcode;
	long				insn_address;
	long				mem_addr;
	long				seq_num;
	} FP_STATE_TYPE;

FP_STATE_TYPE	FP_State[NUM_FP_REGS_];

#define PARAM_TABLE_TYPE_SIZE	16
typedef struct {
	IEEE_UNION_	data_val[PARAM_TABLE_TYPE_SIZE];
	long		hits[PARAM_TABLE_TYPE_SIZE];
	IEEE_UNION_	delta_from_prior;
	long		delta_hits;
	long		delta_restarts;
	} PARAM_TABLE_TYPE;

#define	PROC_TABLE_TYPE_SIZE	10000
#define	PROC_NAME_SIZE		64
typedef struct {
	char		ProcName[PROC_NAME_SIZE];
	char		ProcFileName[PROC_NAME_SIZE];
	long		StartAddr;
	long		counter;
	} PROC_TABLE_TYPE;

	/*
	 * Simple path construction data types
	 */
typedef struct {
	union {
	    long		block_id_long;
	    int			block_id_int[2];
	    unsigned short	block_id_ushort[4];
	    } u;
	} PATH_BLOCK_DATA;

typedef struct SIMPLE_PATH_MULTI_ELT_TYPE {
	long		creation_epoch;
	long		last_used_epoch;
	long		use_count;
	void		*next_path;	/* to next simple path */
	union {
	    PATH_BLOCK_DATA	*first_block;
	    long		block_id_long;
	    int			block_id_int[2];
	    unsigned short	block_id_ushort[4];
	    } dop;	// dop == data overloaded pointer
	} SIMPLE_PATH_MULTI_ELT_TYPE;

typedef struct SIMPLE_PATH_ONE_ELT_TYPE {
	long		creation_epoch;
	long		last_used_epoch;
	long		use_count;
	void		*next_path;	/* to next simple path */
	PATH_BLOCK_DATA	elt_one;
	} SIMPLE_PATH_ONE_ELT_TYPE;

typedef struct SIMPLE_PATH_TWO_ELT_TYPE {
	long		creation_epoch;
	long		last_used_epoch;
	long		use_count;
	void		*next_path;	/* to next simple path */
	PATH_BLOCK_DATA	elt_one;
	PATH_BLOCK_DATA	elt_two;
	} SIMPLE_PATH_TWO_ELT_TYPE;

	/* For most programs, 50000 is sufficient.  126.gcc needs more */
	/* PovRay needs more than gcc */
#define	BLOCK_TABLE_TYPE_SIZE	150000
typedef struct {
	long	ProcId;
	long	Num_Insns;
	long	Block_Addr;
	long	StartLineNum;
	long	EndLineNum;
	} BLOCK_TABLE_TYPE;

	/*
	 * For simple block node counts
	 */
typedef struct {
	long	counter;
	} NODE_TABLE_TYPE;

/*
 * States for replay trap detection
 */
enum MemoryState {
	LOAD_STATE,
	STORE_STATE,
	MEM_ERROR_STATE
	};

/*
 * Structures for Atom-izing time data collection
 */
static long ProcSequenceNum;
static PROC_TABLE_TYPE ProcTable[PROC_TABLE_TYPE_SIZE];
static long BlockSequenceNum;
static BLOCK_TABLE_TYPE BlockTable[BLOCK_TABLE_TYPE_SIZE];

/*
 * Structures for execution time data collection
 */

FILE	*Inst_Logfile;

/*
 * Macros (YUK) and constants for extracting the LIT field from an
 * operate instruction, since the ATOM people didn't deem it important
 * enough to provide this feature.  (Grumble)
 */

#define	OPCODE_MASK		0xfc000000
#define	OPERATE_MASK		0xfc001fe0
#define	JUMP_MASK		0xfc00c000
#define LITERAL_REGISTER_MASK	0x1000
#define LITERAL_FIELD_MASK	0x1fe000
#define LITERAL_SHIFT_COUNT	13
#define LITERAL_BIT_MASK	0xff

#define IS_LITERAL_OPERATE_(OPCODE32)	\
	(OPCODE32 & LITERAL_REGISTER_MASK)
#define LITERAL_VALUE_(OPCODE32)	\
	((OPCODE32 >> LITERAL_SHIFT_COUNT) & LITERAL_BIT_MASK)

